__doc__ = """Audio product module."""
__version__ = '$Revision: 0.0.4 $'[11:-2]  

from Globals import HTMLFile, MessageDialog
from OFS.PropertyManager import PropertyManager
from AccessControl.Role import RoleManager
from OFS.Cache import Cacheable
from OFS.SimpleItem import Item_w__name__
from OFS.Image import *
from mp3info import *

manage_addAudioForm = HTMLFile('audioAdd', globals())

def manage_addAudio(self, id, file='', title='', description='',
                       content_type='', precondition='', REQUEST=None):
    """Add some Audio to a Folder"""

    id, title = cookId(id, title, file)

    self = self.this()
    self._setObject(id, Audio(id=id, file=file, title=title, description=description,
                    content_type=content_type, precondition=precondition))
    self._getOb(id).manage_upload(file)
  
    if REQUEST is not None:
        return self.manage_main(self, REQUEST)


class Audio(File):
    """ Audio: A Filetype with stuff to extract metadata """
    
    meta_type = 'Audio'

    manage_options = (
        ({'label': 'Edit',        'action': 'manage_main'},
        {'label': 'Upload',      'action': 'manage_uploadForm'},
        {'label': 'View',        'action': ''},  #defaults to index_html
        )
        + PropertyManager.manage_options
        + RoleManager.manage_options
        + Item_w__name__.manage_options
        + Cacheable.manage_options
        )

    _properties = ({'id':'title',               'type':'string', 'mode': 'w'},
                 {'id':'description',           'type':'text', 'mode': 'w'},
                 {'id':'content_type',          'type':'string', 'mode': 'w'},
                 {'id':'size',                      'type':'string', 'mode': ''},
                 {'id':'streamurls',       'type':'lines', 'mode': 'w'},
                 {'id':'songname',       'type':'string', 'mode': 'w'},
                 {'id':'artist',             'type':'string', 'mode': 'w'},
                 {'id':'album',          'type':'string', 'mode': 'w'},
                 {'id':'year',           'type':'string', 'mode': 'w'},
                 {'id':'genre',          'type':'selection', 'mode': 'w', 'select_variable':'genre_list'},
#                 {'id':'filesize',       'type':'int', 'mode': ''},
                 {'id':'mpeg',        'type':'string', 'mode': ''},
                 {'id':'layer',       'type':'string', 'mode': ''},
                 {'id':'protection',  'type':'string', 'mode': ''},
                 {'id':'bitrate',         'type':'string', 'mode': ''},
                 {'id':'frequency',   'type':'string', 'mode': ''},
                 {'id':'channel_mode', 'type':'string', 'mode': ''},
                 {'id':'copyright',   'type':'string', 'mode': ''},
                 {'id':'original',    'type':'string', 'mode': ''},
                 {'id':'length',      'type':'string', 'mode': ''},
                 {'id':'framelength',     'type':'string', 'mode': ''},
                 {'id':'filetype',     'type':'string', 'mode': ''},
#                 {'id':'genre_list',     'type':'lines', 'mode': ''},
                 )

    __ac_permissions__ = (
        ('View management screens', 
            ('manage', 'manage_main', 'manage_uploadForm')),
        ('Change Audio',        
            ('manage_edit', 'manage_upload')),
        ('View Audio',
            ('index_html', 'view_image_or_file', 'getSize',
             'getContentType', '')),
        ('Manage properties', ('manage_addProperty',
                             'manage_editProperties',
                             'manage_delProperties',
                             'manage_changeProperties',)),
        )

    def __init__(self, id, file='',  title='', description = '', precondition='',
                 content_type = '',):
        """ Initialize an Audio """

        #first init mp3 meta data...
        self.mp3 = mp3info()
        
        #filetype descriptors
        self.filetype_not_recognized = 'not recognized'
        self.filetype_mp3 = 'mp3 file'
        self.filetype_ra = 'realAudio file' #future
        self.filetype_wav = 'Wave file' #future
        self.filetype_aif = 'AIFF file' #future

        
        #init properties
        self.title = title
        self.description = description
        self.content_type = content_type
        self.precondition=precondition
        self.size = ''
        self.streamurls = []
        self.songname = ''
        self.artist = ''
        self.album = ''
        self.year = ''
        self.genre = ''
        self.genre_list = self.mp3.get_genre_list()
        self.filesize = 0
        self.mpeg = ''
        self.layer = ''
        self.protection = ''
        self.bitrate = ''
        self.frequency = ''
        self.channel_mode = ''
        self.copyright = ''
        self.original = ''
        self.length = ''
        self.framelength = ''
        self.filetype = self.filetype_not_recognized

        #then init file stuff, which because of method below, updates mp3 meta data
        File.__init__(self, id, title, file, content_type, precondition)


    manage_editForm = HTMLFile('audioEdit', globals())
    manage = manage_main = manage_editForm
    manage_uploadForm = HTMLFile('audioUpload', globals())

#replace method with versions which filter the data to
# and update metadata mp3 tags
# this is called only when uploading or adding a new file

    def update_data(self, data, content_type=None, size=None):
        if content_type is not None: self.content_type=content_type
        if size is None: size=len(data)
        self.size=size
        self.data = data    #assign here, since we might destroy data below
        try:
            if type(data) == type(''):
                self.mp3(data)
            else: #Pdata() -- see Image.py
                temp = []
                while data is not None:
                    temp.append(data.data)
                    data=data.next
                self.mp3(string.join(temp,''))
                del temp
            #set properties
            self.songname = self.mp3.id3['songname']
            self.artist = self.mp3.id3['artist']
            self.album = self.mp3.id3['album']
            self.year = self.mp3.id3['year']
            self.genre = self.mp3.id3['genre']
            self.filesize = self.mp3.id3['filesize']
            self.mpeg = self.mp3.id3['mpeg']
            self.layer = self.mp3.id3['layer']
            self.protection = self.mp3.id3['protection']
            self.bitrate = self.mp3.id3['bitrate']
            self.frequency = self.mp3.id3['frequency']
            self.channel_mode = self.mp3.id3['channel_mode']
            self.copyright = self.mp3.id3['copyright']
            self.original = self.mp3.id3['original']
            self.length = self.mp3.id3['length']
            self.framelength = self.mp3.id3['framelength']
            self.filetype = self.filetype_mp3
            #set normal file properties from song data
            self.title = self.songname
            self.description = self.artist + '\n' \
                 + self.album + '\n' + self.year + '\n' + self.copyright 
        except (InvalidHeaderError, NoHeaderError, NoTagError), error:
            #this is where you could look for other types of audio data...
            self.filetype = self.filetype_not_recognized
        self.mp3.free_file()
        self.ZCacheable_invalidate()

# update internal tag dict from propertysheet values
    def _update_mp3tags(self):
        self.mp3.id3['songname'] = self.songname
        self.mp3.id3['artist'] =  self.artist
        self.mp3.id3['album'] = self.album
        self.mp3.id3['year'] = self.year
        self.mp3.id3['genre'] = self.genre
        #create genre_no from genre if possible
        try:
            self.mp3.id3['genre_no'] = self.mp3.get_genre_list().index(self.genre)
        except:
            self.mp3.id3['genre_no'] = 255


# replace manage_edit to allow editing mp3 tags


    def manage_edit(self, title='' ,content_type='',precondition='', description='', REQUEST=None):
        """
        Changes the  attributes of the Audio File.
        """
        self.title=str(title)
        self.content_type=str(content_type)
        self.description = description
        if precondition: self.precondition=str(precondition)
        elif self.precondition: del self.precondition
        self.ZCacheable_invalidate()
        if REQUEST:
            message="Saved changes!"+self.songname+self.artist
            return self.manage_main(self,REQUEST,manage_tabs_message=message)

#replace index method to output id3 tags at the end of data output
#since this is the only place the file is read, it's a good place to
#update the internal mp3 tag dict from the propertysheet values

    def index_html(self, REQUEST, RESPONSE):
        """
        The default view of the contents of a File or Image.

        Returns the contents of the file or image.  Also, sets the
        Content-Type HTTP header to the objects content type.
        """
        # HTTP If-Modified-Since header handling.
        header=REQUEST.get_header('If-Modified-Since', None)
        if header is not None:
            header=string.split(header, ';')[0]
            # Some proxies seem to send invalid date strings for this
            # header. If the date string is not valid, we ignore it
            # rather than raise an error to be generally consistent
            # with common servers such as Apache (which can usually
            # understand the screwy date string as a lucky side effect
            # of the way they parse it).
            try:    mod_since=long(DateTime(header).timeTime())
            except: mod_since=None
            if mod_since is not None:
                if self._p_mtime:
                    last_mod = long(self._p_mtime)
                else:
                    last_mod = long(0)
                if last_mod > 0 and last_mod <= mod_since:
                    # Set header values since apache caching will return Content-Length
                    # of 0 in response if size is not set here
                    RESPONSE.setHeader('Last-Modified', rfc1123_date(self._p_mtime))
                    RESPONSE.setHeader('Content-Type', self.content_type)
                    RESPONSE.setHeader('Content-Length', self.size)
                    RESPONSE.setStatus(304)
                    return ''

        if self.precondition and hasattr(self,self.precondition):
            # Grab whatever precondition was defined and then 
            # execute it.  The precondition will raise an exception 
            # if something violates its terms.
            c=getattr(self,self.precondition)
            if hasattr(c,'isDocTemp') and c.isDocTemp:
                c(REQUEST['PARENTS'][1],REQUEST)
            else:
                c()
        RESPONSE.setHeader('Last-Modified', rfc1123_date(self._p_mtime))
        RESPONSE.setHeader('Content-Type', self.content_type)
        RESPONSE.setHeader('Content-Length', self.size)

        # Don't cache the data itself, but provide an opportunity
        # for a cache manager to set response headers.
        self.ZCacheable_set(None)

        data=self.data
        
        if self.filetype == self.filetype_mp3:  #not an mp3 file, don't output a bogus tag
            self._update_mp3tags() #update internal tag dict with any propertysheet changes
            if type(data) == type(''): return data[:-128] + self.mp3.mp3_tag()
            #special crap for Pdata type
            while data.next is not None:  #special treatment for last segment
                RESPONSE.write(data.data)
                data=data.next
            RESPONSE.write(data.data[:-128])
            RESPONSE.write(self.mp3.mp3_tag())
            return ''

        else: #future - do other types of tag insertions here
            if type(data) is type(''): return data

            while data is not None:
                RESPONSE.write(data.data)
                data=data.next

            return ''

